# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - FetchMessages Action

    @copyright: 2012, 2013, 2014 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinSupport import getMetadata, writeHeaders, parseDictEntry
from MoinMessage import GPG, MessageInterface
from MoinMessageSupport import MoinMessageAction, \
                               get_signing_users, get_recipient_details, \
                               MoinMessageRecipientError

Dependencies = ['pages']

class FetchMessages(MoinMessageAction):

    "A handler for requests accessing messages."

    def handle_message_content(self, content):

        "Handle the given message 'content'."

        request = self.request

        # Employing an unstandardised content type.

        if not content.get_content_type() == "text/x-moinmessage-fetch":
            writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
            request.write("The content does not appear to be a request for messages.")
            return

        homedir = self.get_homedir()
        if not homedir:
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
            request.write("This site is not configured for this request.")
            return

        gpg = GPG(homedir)

        # Get keys for signing and encrypting.
        # The signing key will be this wiki's signing key for the user
        # requesting the messages.
        # The encryption key will be the key associated with the user requesting
        # the messages, found in the recipients mapping.

        recipient = request.user.name

        signing_users = get_signing_users(request)
        signer = signing_users and signing_users.get(recipient)

        # Get the recipient details.

        try:
            parameters = get_recipient_details(request, recipient, fetching=True)
        except MoinMessageRecipientError, exc:
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
            request.write(exc.message)
            return

        # Obtain commands from the payload, returning a collection of messages
        # and command results.

        commands = content.get_payload(decode=True)
        message = MessageInterface(self.store).execute(commands)

        # Sign and encrypt the message.

        message = message.get_payload()

        if signer:
            message = gpg.signMessage(message, signer)

        message = gpg.encryptMessage(message, parameters["fingerprint"])

        # Write the response.

        writeHeaders(request, "text/x-moinmessage-fetch-response", getMetadata(self.page))
        request.write(message.as_string())

# Action function.

def execute(pagename, request):
    FetchMessages(pagename, request).do_action() # instead of render

# vim: tabstop=4 expandtab shiftwidth=4
